import json
import pandas as pd
import networkx as nx
from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import seaborn as sns

class Data_management():
    def __init__(self, data):
        self.data = data

    def show(self):
        return self.data

    def province_bar(self):
        fig = plt.Figure(figsize=(12, 6), dpi=100)
        ax_bar = fig.add_subplot()
        ax_bar.bar(self.data.จังหวัด.value_counts().index, self.data.จังหวัด.value_counts().values, color='green')
        ax_bar.bar_label(ax_bar.containers[0], labels=self.data.จังหวัด.value_counts().values, rotation=30, fontsize=7)
        tick_labels = self.data.จังหวัด.value_counts().index
        tick_positions = range(len(tick_labels))
        ax_bar.set_xticks(tick_positions)
        ax_bar.set_xticklabels(tick_labels, rotation=45, ha="right", fontsize=7)
        ax_bar.set_xlabel("province")
        ax_bar.set_ylabel("accident")
        ax_bar.set_title("Accident per province")
        fig.tight_layout()
        return fig

    def month_line(self):
        ml = self.data.copy()
        cml = ml.dropna(subset=['วันที่เกิดเหตุ'])
        cml['วันที่เกิดเหตุ'] = pd.to_datetime(cml['วันที่เกิดเหตุ'], format='%d/%m/%Y')
        cml.index = cml['วันที่เกิดเหตุ']
        cml = cml.resample('M').count()
        fig = plt.Figure(figsize=(12, 6), dpi=100)
        ax_line = fig.add_subplot()
        sns.lineplot(data=cml, x=cml.index, y='จังหวัด', ax=ax_line)
        ax_line.set_xticks(cml.index)
        ax_line.set_xticklabels(cml.index.strftime('%Y-%m'), rotation=45, ha="right", fontsize=7)
        ax_line.set_xlabel("month")
        ax_line.set_ylabel("accident")
        ax_line.set_title("Accident per month")
        fig.tight_layout()
        return fig

    def type_accident_pie(self):
        tap = self.data.copy()
        tap = tap.dropna(subset=['ลักษณะการเกิดอุบัติเหตุ'])
        fig = plt.Figure(figsize=(12, 100), dpi=100)
        ax_pie = fig.add_subplot()
        ax_pie.pie(tap['ลักษณะการเกิดอุบัติเหตุ'].value_counts().values,
                   labels=tap['ลักษณะการเกิดอุบัติเหตุ'].value_counts().index,
                   autopct='%1.1f%%')
        ax_pie.legend(loc="upper left", bbox_to_anchor=(-0.8, -0.1, 1, 1))
        ax_pie.set_title("Accident type")
        return fig

    def get_province(self):
        with open('province.json', 'r') as file:
            data = json.load(file)
        return data

    def create_basemap_plot(self, departure_province=None, destination_province=None):
        with open("thailand_province_relations.txt", "r") as file:
            data = file.read().splitlines()

        list_relation = []
        for i in data:
            departure, destinations = i.split(" -> ")
            destinations_list = destinations.split(", ")
            for j in destinations_list:
                list_relation.append((departure, j))

        with open('province.json', 'r') as file:
            data = json.load(file)

        fig = plt.figure(figsize=(8, 6), dpi=100)
        map = Basemap(llcrnrlon=97, llcrnrlat=5, urcrnrlon=106, urcrnrlat=21, resolution='l')

        map.drawcoastlines()
        map.drawcountries()

        G = nx.Graph()

        for province, coordinates in data.items():
            lat, lon = coordinates
            x, y = map(lon, lat)
            G.add_node(province, pos=(x, y))

        for edge in list_relation:
            departure, destination = edge
            departure_coords = data[departure]
            destination_coords = data[destination]
            lat1, lon1 = departure_coords
            lat2, lon2 = destination_coords
            x1, y1 = map(lon1, lat1)
            x2, y2 = map(lon2, lat2)
            G.add_edge(departure, destination, weight=((x1 - x2) ** 2 + (y1 - y2) ** 2) ** 0.5)

        if departure_province and destination_province:
            shortest_path = nx.dijkstra_path(G, departure_province, destination_province)

        # Plot the nodes on the map
        for province, coordinates in data.items():
            lat, lon = coordinates
            x, y = map(lon, lat)
            if departure_province and destination_province and province in shortest_path:
                map.plot(x, y, 'ro', markersize=6)
            else:
                map.plot(x, y, 'bo', markersize=6)
            plt.text(x, y, province, fontsize=8, ha='center', va='bottom')

        for edge in list_relation:
            departure, destination = edge
            departure_coords = data[departure]
            destination_coords = data[destination]
            lat1, lon1 = departure_coords
            lat2, lon2 = destination_coords
            x1, y1 = map(lon1, lat1)
            x2, y2 = map(lon2, lat2)
            if departure_province and destination_province and departure in shortest_path and destination in shortest_path:
                map.plot([x1, x2], [y1, y2], 'r', alpha=0.6)
            else:
                map.plot([x1, x2], [y1, y2], 'gray', alpha=0.6)

        plt.title("Province relations")

        return fig
