import json
import pandas as pd
import networkx as nx
import matplotlib.pyplot as plt
import seaborn as sns


class Data_management():
    def __init__(self, data):
        self.data = data

    def show(self):
        return self.data

    def province_bar(self):
        fig = plt.Figure(figsize=(12, 6), dpi=100)
        ax_bar = fig.add_subplot()
        ax_bar.bar(self.data.จังหวัด.value_counts().index, self.data.จังหวัด.value_counts().values, color='green')
        ax_bar.bar_label(ax_bar.containers[0], labels=self.data.จังหวัด.value_counts().values, rotation=30, fontsize=7)
        tick_labels = self.data.จังหวัด.value_counts().index
        tick_positions = range(len(tick_labels))
        ax_bar.set_xticks(tick_positions)
        ax_bar.set_xticklabels(tick_labels, rotation=45, ha="right", fontsize=7)
        ax_bar.set_xlabel("province")
        ax_bar.set_ylabel("accident")
        ax_bar.set_title("Accident per province")
        fig.tight_layout()
        return fig

    def month_line(self):
        ml = self.data.copy()
        cml = ml.dropna(subset=['วันที่เกิดเหตุ'])
        cml['วันที่เกิดเหตุ'] = pd.to_datetime(cml['วันที่เกิดเหตุ'], format='%d/%m/%Y')
        cml.index = cml['วันที่เกิดเหตุ']
        cml = cml.resample('M').count()
        fig = plt.Figure(figsize=(12, 6), dpi=100)
        ax_line = fig.add_subplot()
        sns.lineplot(data=cml, x=cml.index, y='จังหวัด', ax=ax_line)
        ax_line.set_xticks(cml.index)
        ax_line.set_xticklabels(cml.index.strftime('%Y-%m'), rotation=45, ha="right", fontsize=7)
        ax_line.set_xlabel("month")
        ax_line.set_ylabel("accident")
        ax_line.set_title("Accident per month")
        fig.tight_layout()
        return fig

    def type_accident_pie(self):
        tap = self.data.copy()
        tap = tap.dropna(subset=['ลักษณะการเกิดอุบัติเหตุ'])
        fig = plt.Figure(figsize=(12, 100), dpi=100)
        ax_pie = fig.add_subplot()
        ax_pie.pie(tap['ลักษณะการเกิดอุบัติเหตุ'].value_counts().values,
                   labels=tap['ลักษณะการเกิดอุบัติเหตุ'].value_counts().index,
                   autopct='%1.1f%%')
        ax_pie.legend(loc="upper left", bbox_to_anchor=(-0.8, -0.1, 1, 1))
        ax_pie.set_title("Accident type")
        return fig

    def get_province(self):
        with open('province.json', 'r') as file:
            data = json.load(file)
            return data

    def create_basemap_plot(self, source_province=None, destination_province=None):
        with open("thailand_province_relations.txt", "r") as file:
            data = file.read().splitlines()

        list_relation = []
        for i in data:
            departure, destinations = i.split(" -> ")
            destinations_list = destinations.split(", ")
            for j in destinations_list:
                list_relation.append((departure, j))

        with open('province.json', 'r') as file:
            data = json.load(file)

        G = nx.Graph()

        for province, coordinates in data.items():
            G.add_node(province, pos=coordinates)

        for edge in list_relation:
            departure, destination = edge
            G.add_edge(departure, destination)

        fig = plt.figure(figsize=(8, 6), dpi=100)
        ax = fig.add_subplot()

        pos = nx.get_node_attributes(G, 'pos')

        if source_province and destination_province:
            shortest_path = nx.dijkstra_path(G, source_province, destination_province)

        for node, coordinates in pos.items():
            if source_province and destination_province and node in shortest_path:
                ax.plot(coordinates[0], coordinates[1], 'ro', markersize=6)
            else:
                ax.plot(coordinates[0], coordinates[1], 'bo', markersize=6)
            ax.text(coordinates[0], coordinates[1], node, fontsize=8, ha='center', va='bottom')

        for edge in list_relation:
            departure, destination = edge
            departure_coords = pos[departure]
            destination_coords = pos[destination]
            if source_province and destination_province and departure in shortest_path and destination in shortest_path:
                ax.plot([departure_coords[0], destination_coords[0]], [departure_coords[1], destination_coords[1]], 'r',
                        alpha=0.6)
            else:
                ax.plot([departure_coords[0], destination_coords[0]], [departure_coords[1], destination_coords[1]], 'gray',
                        alpha=0.6)

        plt.title("Province Connections")

        return fig
